unit CameraCGWMainUnit;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  FMX.Controls.Presentation, FMX.StdCtrls,
  FMX.DialogService,  System.Permissions,
  PeteACameraUnit,
  System.IOUtils, FMX.Memo.Types, FMX.ScrollBox, FMX.Memo, System.Actions,
  FMX.ActnList, FMX.StdActns, FMX.MediaLibrary.Actions;

{*
As well as the permissions in Project/Options/Uses Permissions,
you will need
Project/Options/Entitlement List/Secure File Sharing ticked.
*}

type
  TCameraCGWMainForm = class(TForm)
    ButtonStart: TButton;
    ButtonExit: TButton;
    Memo1: TMemo;

    procedure GetPermissions;
    procedure PhotoFinished;
    function IntTo2Digits(number : integer) : string;

    procedure ButtonExitClick(Sender: TObject);
    procedure ButtonStartClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  CameraCGWMainForm: TCameraCGWMainForm;
  PermissionsOK : boolean;

implementation

{$R *.fmx}


procedure TCameraCGWMainForm.FormCreate(Sender: TObject);
begin
   Memo1.Lines.Clear;
   GetPermissions;
   PermissionsOK := false;
end;



procedure tCameraCGWMainForm.GetPermissions;
// Can't have this as boolean, as the permission getting stuff seems to be run in a
// different thread.  Hence the PermissionsOK variable.
var
   AndroidPermission : string;
   PermissionAccessFineLocation : string;
   PermissionAccessCoarseLocation : string;
   PermissionAccessBackgroundLocation : string;
   PermissionAccessReadExternalStorage : string;
   PermissionAccessWriteExternalStorage : string;
   PermissionCamera : string;

   Permissions: tArray<system.string>;
   sPermissions : array of string;

   s : string;
   ok : boolean;
begin
   s := '';

   AndroidPermission := 'android.permission.';
   PermissionAccessFineLocation := 'ACCESS_FINE_LOCATION';
   PermissionAccessCoarseLocation := 'ACCESS_COARSE_LOCATION';
   PermissionAccessBackgroundLocation := 'ACCESS_BACKGROUND_LOCATION';
   PermissionAccessReadExternalStorage := 'READ_EXTERNAL_STORAGE';
   PermissionAccessWriteExternalStorage := 'WRITE_EXTERNAL_STORAGE';
   PermissionCamera := 'CAMERA';

   setlength(Permissions, 3);
   setlength(sPermissions, 3);
   sPermissions[0] := PermissionAccessReadExternalStorage;
   sPermissions[1] := PermissionAccessWriteExternalStorage;
   sPermissions[2] := PermissionCamera;
   Permissions[0] := AndroidPermission +  PermissionAccessReadExternalStorage;
   Permissions[1] := AndroidPermission +  PermissionAccessWriteExternalStorage;
   Permissions[2] := AndroidPermission +  PermissionCamera;

   PermissionsService.RequestPermissions(Permissions,
      procedure(const APermissions: TClassicStringDynArray;
                const AGrantResults: TClassicPermissionStatusDynArray)
      var
         i : integer;
      begin
         if length(AGrantResults) <= 0 then
         begin
            ok := false;
            s := sPermissions[0] + ', ' + sPermissions[1] + ', ' + sPermissions[2];
         end
         else
         begin
            ok := true;
            for i := 0 to length(AGrantResults) - 1 do
               if AGrantResults[i] <> tPermissionStatus.Granted then
               begin
                  ok := false;
                  if s <> '' then s := s + ', ';
                  s := s + sPermissions[i];
               end;
         end;
            if not ok then
               TDialogService.ShowMessage('Permissions required for this programme, ' + s + ' denied');
         PermissionsOK := ok;
      end
      );
end;




procedure TCameraCGWMainForm.ButtonExitClick(Sender: TObject);
begin
   Application.Terminate;
end;

procedure TCameraCGWMainForm.ButtonStartClick(Sender: TObject);
var
  PhotoDirectory : string;
  Year, Month, Day, Hour, Minutes, Seconds, mSec : word;
  HourTaken : string;
  T : tDateTime;
  sl : tStringList;
begin
   //Disable everything to start fresh - or can get some funny errors to do with
   // Java and preview settings, or things with zero values, etc.

   PeteACameraForm.CameraComponent1.Active := false;
   PeteACameraForm.TakePhotoFromCameraAction1.Enabled := false;

   T := Now;

   DecodeDate(T, Year, Month, Day);
   DecodeTime(T, Hour, Minutes, Seconds, mSec);

   HourTaken := IntToStr(Year) + IntTo2Digits(Month) + IntTo2Digits(Day) + ' ' +
      IntTo2Digits(Hour);

   PhotoDirectory := System.IOUtils.tPath.GetSharedDocumentsPath + '/Test/';
   tDirectory.CreateDirectory(PhotoDirectory);

   PhotoDirectory := PhotoDirectory + HourTaken + '/';
   tDirectory.CreateDirectory(PhotoDirectory);

   With PeteACameraForm do
   begin
     SetPhotoType('jpg');       //For CameraComponent
     SetPhotoQuality('High');
     SetTorchOption('On');

     SetPhotoDirectory(PhotoDirectory);  //For both camera types
     WhenFinishedPhotos(PhotoFinished);
   end;

  PeteACameraForm.ShowModal(procedure (AResult: TModalResult)
     var i : integer;
     begin
        sl := PeteACameraForm.GetNamesOfPhotosTaken;
        Memo1.Lines.Clear;
        for i := 0 to sl.Count - 1 do Memo1.Lines.Add(sl[i]);
        sl.Destroy;
     end);
end;


procedure tCameraCGWMainForm.PhotoFinished;  // Called when user has finished taking piccies
var
  sl : tStringList;
  i : integer;
begin
   sl := PeteACameraForm.GetNamesOfPhotosTaken;
   Memo1.Lines.Clear;
   if sl.Count > 0 then
      for i := 0 to sl.Count - 1 do Memo1.Lines.Add(sl[i]);
   sl.Destroy;
end;



function tCameraCGWMainForm.IntTo2Digits(number : integer) : string;
var
   s : string;
begin
   s := IntToStr((number + 100) mod 1000);
   Result := s[2] + s[3];
end;


end.
