unit PeteACameraUnit;
{ PeteA is a programme for tracking little furry critters around the Australian
bush.  Due to android restrictions from android 10, it became necessary to be able
to take piccies from the app itself, or waste up to a minute waiting for location
to come back after using the camera - a real curse if you are tracking from a
plane!

It has things which are handy for the PeteA programme, which anyone else using
for his/her own purposes is welcome to chop and change!

The bit that will strike people as odd is the use of tinted buttons to make
edit boxes obvious.  I tried to use drawing on an image behind the edit boxes,
but while it worked well on windows (and was unnecessary there!) it produced
horrible results on FMX.  This technique was arrived at after huge numbers of
hours of experimentation!

Version of 29-May-22
}

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  FMX.Controls.Presentation, FMX.StdCtrls, FMX.Objects, System.Actions,
  FMX.ActnList, FMX.Media, uSettings, FMX.Edit,
  System.IOUtils, FMX.StdActns, FMX.MediaLibrary.Actions;

type
  TPeteACameraForm = class(TForm)
    Image1: TImage;
    ButtonOnAction: TButton;
    ButtonOff: TButton;
    ButtonExit: TButton;
    ButtonSave: TButton;
    CameraComponent1: TCameraComponent;
    ActionList1: TActionList;
    ButtonSettings: TButton;
    ButtonFileName: TButton;
    EditFilePrefix: TEdit;
    EditFileSuffix: TEdit;
    EditFileExt: TEdit;
    ButtonPseudo1: TButton;
    ButtonPseudo2: TButton;
    ButtonPseudo3: TButton;
    TakePhotoFromCameraAction1: TTakePhotoFromCameraAction;
    ButtonOnComponent: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure ButtonOnActionClick(Sender: TObject);
    procedure ButtonOffClick(Sender: TObject);
    procedure ButtonSaveClick(Sender: TObject);
    procedure ButtonExitClick(Sender: TObject);

   procedure CameraComponentSampleBufferReady(Sender: TObject; const ATime: TMediaTime);
   procedure DisplayCameraPreviewFrame;
   procedure actShowSettingsExecute(Sender: TObject);
   procedure EditFileExtClick(Sender: TObject);
   function IntTo2Digits(number : integer) : string;
   function SavePicture : boolean;
   procedure TakePhotoFromCameraAction1DidFinishTaking(Image: TBitmap);
    procedure ButtonOnComponentClick(Sender: TObject);
    procedure FormShow(Sender: TObject);


  private
    { Private declarations }

  public
    { Public declarations }

  published

     procedure SetPhotoDirectory(s : string);
     function SetPhotoType(s : string) : boolean;
     function SetPhotoQuality(s : string) : boolean;
     function SetTorchOption(s : string) : boolean;
     procedure SetPhotoFilenameSuffix(s : string);
     procedure WhenFinishedPhotos(Proc : tProc);
     function GetNamesOfPhotosTaken : tStringList;
  end;

var
   PeteACameraForm: TPeteACameraForm;

   PhotoDirectory : string;
   Year, Month, Day, Hour, Minutes, Seconds, mSec : word;
   HourTaken, TimeTaken : string;

   QualityTypes : array of string = ['Photo', 'High', 'Medium', 'Low'];
   QualityValues : array of TVideoCaptureQuality = [TVideoCaptureQuality.PhotoQuality,
                                      TVideoCaptureQuality.HighQuality,
                                      TVideoCaptureQuality.MediumQuality,
                                      TVideoCaptureQuality.LowQuality];
   TorchOptions : array of string = ['On', 'Off', 'Auto'];
   TorchValues : array of tTorchMode = [tTorchMode.ModeOn, tTorchMode.ModeOff, tTorchMode.ModeAuto];

   WhenFinishedProc : tProc;

   PhotoList : tStringList; //List of names of photos taken.

   tempint1, tempint2, tempint3, tempint4 : integer;     //Debugging aid

implementation

{$R *.fmx}

uses CameraCGWMainUnit;

// Routines for communication with the calling programme.  These should be done,
// then the PeteACameraForm.Show.

// SetPhotoDirectory is self-explanatory.
// SetPhotoFilenameSuffix - the default filename is <YYYYMMDD> + this suffix if specified.
// WhenFinishedPhotos allows the user to specify a routine to be called when the user
//   has finished taking the photos.
// SetPhotoType, SetPhotoQuality, and SetTorchOption are for CameraComponent.
// GetNamesOfPhotosTaken provides a string list of the names of photos taken since
//   the last call to GetNamesOfPhotosTaken (or from the start of the programme).

procedure tPeteACameraForm.SetPhotoDirectory(s: string);
begin
  PhotoDirectory := s;
end;

procedure tPeteACameraForm.SetPhotoFilenameSuffix(s : string);
begin
   PeteACameraForm.EditFilePrefix.Text := s;
end;

procedure tPeteACameraForm.WhenFinishedPhotos(Proc: tProc);
begin
   WhenFinishedProc := Proc;
end;

function tPeteACameraForm.SetPhotoType(s: string): Boolean;
var
  ss : string;
begin
  ss := '';

  if lowercase(s) = 'jpg' then ss := '.jpg';
  if lowercase(s) = 'png' then ss := '.png';

  if ss <> '' then Result := true
  else
  begin
     Result := false;
     ss := '.jpg';
  end;

  EditFileExt.Text := ss;
end;


function tPeteACameraForm.SetPhotoQuality(s : string) : boolean;
var
   i : integer;
  Setting: TVideoCaptureSetting;
begin
   i := high(QualityTypes);
   while i <= high(QualityTypes) do
   begin
      if lowercase(s) = lowercase(QualityTypes[i]) then break;
      dec(i);
      if i < 0 then break;
   end;

   if i >= 0 then
   begin
      Result := true;
      CameraComponent1.Quality := TVideoCaptureQuality(QualityValues[i]);
      uSettings.SettingsForm.lbiQuality.ItemData.Detail := QualityTypes[i];

      Setting := CameraComponent1.CaptureSetting;
      uSettings.SettingsForm.lbiResolution.ItemData.Detail :=
         Format('%d x %d x %f', [Setting.Width, Setting.Height, Setting.FrameRate]);
   end
   else
   begin
      Result := false;
   end;

end;

function tPeteACameraForm.SetTorchOption(s : string) : boolean;
var
  i : integer;
  SavedCameraActive : boolean;
begin
   i := high(TorchOptions);
   while i <= high(TorchOptions) do
   begin
      if lowercase(s) = lowercase(TorchOptions[i]) then break;
      dec(i);
      if i < 0 then break;
   end;

   SavedCameraActive := CameraComponent1.Active;
   CameraComponent1.Active := true;  //If you don't do this, can get some funny prangs.

   if i >= 0 then
   begin
      Result := true;
      CameraComponent1.TorchMode := TorchValues[i];
      uSettings.SettingsForm.lbiTorchMode.ItemData.Detail := TorchOptions[i];
   end
   else
   begin
      Result := false;
   end;

   CameraComponent1.Active := SavedCameraActive;
end;



// And at the end, when the user may like a list of the photos taken since the
// last call to GetNamesOfPhotosTaken (or the start of the programme):

function tPeteACameraForm.GetNamesOfPhotosTaken: TStringList;
var
   i : integer;
begin
   Result := tStringList.Create;
   i := PhotoList.Count;
   if i > 0 then for i := 0 to PhotoList.Count - 1 do
      Result.Add(PhotoList[i]);
   PhotoList.Clear;
end;



//------------------------------------------------------------------------------



procedure TPeteACameraForm.FormCreate(Sender: TObject);
begin
   EditFilePrefix.Text := '';
   EditFileSuffix.Text := '';
   PhotoList := tStringList.Create;
end;


//Put in current time, though this will be changed when he actually saves the file.

procedure TPeteACameraForm.FormShow(Sender: TObject);
var
   T : tDateTime;
begin
   T := Now;
   DecodeTime(T, Hour, Minutes, Seconds, mSec);
   TimeTaken :=  IntTo2Digits(Hour) + IntTo2Digits(Minutes) + IntTo2Digits(Seconds);
   EditFilePrefix.Text := TimeTaken;
end;




function tPeteACameraForm.IntTo2Digits(number : integer) : string;
var
   s : string;
begin
   s := IntToStr((number + 100) mod 1000);
   Result := s[2] + s[3];
end;



procedure TPeteACameraForm.ButtonExitClick(Sender: TObject);
begin
   CameraComponent1.Active := false;
   TakePhotoFromCameraAction1.Enabled := false; //This probably not needed.
   Close;
end;

procedure TPeteACameraForm.ButtonOffClick(Sender: TObject);
begin
   CameraComponent1.Active := false;
   TakePhotoFromCameraAction1.Enabled := false; //This probably not needed.
end;


procedure TPeteACameraForm.ButtonOnActionClick(Sender: TObject);
begin
   CameraComponent1.Active := false;

   TakePhotoFromCameraAction1.MaxHeight := 8000;    //These can be set up in Object Inspector
   TakePhotoFromCameraAction1.MaxWidth := 8000;

   TakePhotoFromCameraAction1.Enabled := true;
   TakePhotoFromCameraAction1.Execute;
end;


procedure TPeteACameraForm.ButtonOnComponentClick(Sender: TObject);
begin
   TakePhotoFromCameraAction1.Enabled := false;

   CameraComponent1.Active := true;  //Max parameters set up before showing this form.
end;



procedure TPeteACameraForm.ButtonSaveClick(Sender: TObject);
var
   SavedCameraActive: Boolean;
   SavedActionActive : Boolean;
begin
   SavedCameraActive := CameraComponent1.Active;
   SavedActionActive := TakePhotoFromCameraAction1.Enabled;

   CameraComponent1.Active := false;
   TakePhotoFromCameraAction1.Enabled := false;

   SavePicture;   //Has an error return if exception occurs - not used currently.

   CameraComponent1.Active := SavedCameraActive;
   TakePhotoFromCameraAction1.Enabled := SavedActionActive;
end;


// Give the user the options for setting parameters for the CameraComponent.
// Unfortunately these do not go across to the TakePhoeoFromCamereAction!

procedure tPeteACameraForm.actShowSettingsExecute(Sender: TObject);
var
  SavedCameraActive: Boolean;
begin
  SavedCameraActive := CameraComponent1.Active;
  SettingsForm.Camera := CameraComponent1;
  CameraComponent1.Active := False;

  SettingsForm.ShowModal(procedure (AResult: TModalResult)
     begin
        CameraComponent1.Active := SavedCameraActive;
     end);
end;


//These three are required for the TakePhotoCameraAction code:

procedure tPeteACameraForm.CameraComponentSampleBufferReady(Sender: TObject; const ATime: TMediaTime);
begin
   TThread.Synchronize(TThread.CurrentThread, DisplayCameraPreviewFrame);
end;

procedure tPeteACameraForm.DisplayCameraPreviewFrame;
begin
   CameraComponent1.SampleBufferToBitmap(Image1.Bitmap, True);
end;

procedure TPeteACameraForm.TakePhotoFromCameraAction1DidFinishTaking(
  Image: TBitmap);
begin
   PeteACameraForm.Image1.Bitmap.Assign(Image);
end;

//-------------------------------------------------------


//Select desired picture file extension

procedure TPeteACameraForm.EditFileExtClick(Sender: TObject);
begin
   if EditFileExt.Text = '.jpg' then EditFileExt.Text := '.png'
   else EditFileExt.Text := '.jpg';
end;


//Save the picture - same techniques for both camera types.

function TPeteACameraForm.SavePicture : boolean;
var
   m, i : integer;
   s : string;
   T : tDateTime;
begin
   //Save the file - give it a unique filename for this subdirectory

   T := Now;
   DecodeTime(T, Hour, Minutes, Seconds, mSec);
   TimeTaken :=  IntTo2Digits(Hour) + IntTo2Digits(Minutes) + IntTo2Digits(Seconds);

   if EditFileSuffix.Text <> '' then s := TimeTaken + ' ' + EditFileSuffix.Text
   else s := TimeTaken;

   //Assume that there is only one Bitmap in our collection, as if we use
   //   Image1.Bitmap.SaveToFile(WorkingDirectory + s + '.' + EditExt.Text);
   //   it prangs (zero length file if .jpg extension used) on the Samsung S22 phone.

   m := Image1.MultiResBitmap.Count;
   i := 0;

   if m > 0 then
   begin
      for i := 0 to m - 1 do
      begin
         if round(Image1.MultiResBitmap.Items[i].Bitmap.Width) > 0 then break;
      end;
   end;

   s := PhotoDirectory + s + '.' + EditFileExt.Text;  //Full file name
   try
      Image1.MultiResBitmap.Items[i].Bitmap.SaveToFile(s);
      Result := true;
   except
      Result := false;
   end;

   if Result then PhotoList.Add(s)
   else PhotoList.Add('Error saving ' + s);
end;





procedure TPeteACameraForm.FormResize(Sender: TObject);
begin
   ButtonOnAction.Width := ClientWidth / 4 - 3;
   ButtonOnComponent.Width := ButtonOnAction.Width;
   ButtonSettings.Width := ButtonOnAction.Width;
   ButtonOff.Width := ButtonOnAction.Width;

   ButtonOnACtion.Position.X := 0;
   ButtonOnAction.Position.Y := 0;
   ButtonOnComponent.Position.X := ClientWidth / 4;
   ButtonOnComponent.Position.Y := 0;
   ButtonSettings.Position.X := ClientWidth / 2;
   ButtonSettings.Position.Y := 0;
   ButtonOff.Position.X := (3 * ClientWidth) / 4;
   ButtonOff.Position.Y := 0;

   ButtonFileName.Position.X := 1;
   ButtonFileName.Position.Y := ButtonOnAction.Height + 3;
   ButtonFileName.Width := ClientWidth - 2;

   ButtonFileName.TintColor := tAlphaColorRec.LightCyan;
   ButtonPseudo1.TintColor := tAlphaColorRec.LightCyan;
   ButtonPseudo2.TintColor := tAlphaColorRec.LightCyan;
   ButtonPseudo3.TintColor := tAlphaColorRec.LightCyan;

   EditFilePrefix.Position.X := 1;
   EditFilePrefix.Position.Y := ButtonFileName.Position.Y + ButtonFileName.Height + 3;
   EditFileSuffix.Position.X := (ClientWidth - EditFileSuffix.Width) / 2;
   EditFileSuffix.Position.Y := EditFilePrefix.Position.Y;
   EditFileExt.Position.X := ClientWidth - EditFileExt.Width - 2;
   EditFileExt.Position.Y := EditFilePrefix.Position.Y;

   ButtonPseudo1.Height := EditFilePrefix.Height + 2;
   ButtonPseudo1.Width := EditFilePrefix.Width + 2;
   ButtonPseudo1.Position.X := EditFilePrefix.Position.X - 1;
   ButtonPseudo1.Position.Y := EditFilePrefix.Position.Y - 1;

   ButtonPseudo2.Height := EditFileSuffix.Height + 2;
   ButtonPseudo2.Width := EditFileSuffix.Width + 2;
   ButtonPseudo2.Position.X := EditFileSuffix.Position.X - 1;
   ButtonPseudo2.Position.Y := EditFileSuffix.Position.Y - 1;

   ButtonPseudo3.Height := EditFileExt.Height + 2;
   ButtonPseudo3.Width := EditFileExt.Width + 2;
   ButtonPseudo3.Position.X := EditFileExt.Position.X - 1;
   ButtonPseudo3.Position.Y := EditFileExt.Position.Y - 1;


   ButtonSave.Position.X := 0;
   ButtonSave.Position.Y := ClientHeight - ButtonSave.Height;

   ButtonExit.Position.X := ClientWidth - ButtonExit.Width;
   ButtonExit.Position.Y := ClientHeight - ButtonExit.Height;

   Image1.Position.X := 0;
   Image1.Position.Y := EditFilePrefix.Position.Y - 1;
   Image1.Height := ClientHeight - ButtonOnAction.Height - ButtonSave.Height - 6;
   Image1.Width := ClientWidth;
end;

end.
